import os


DEFAULT_SERVER_CONFIG_PATH = os.path.expanduser('~/.askanna.yml')
DEFAULT_SERVER_REMOTE = 'https://beta-api.askanna.eu'
DEFAULT_SERVER_UI = 'https://beta.askanna.eu'

DEFAULT_PROJECT_TEMPLATE = 'https://gitlab.com/askanna/project-templates/blanco-template.git'

PYPI_PROJECT_URL = 'https://pypi.org/pypi/askanna/json'
